"""
Type annotations for iotthingsgraph service client paginators.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from boto3.session import Session

    from types_boto3_iotthingsgraph.client import IoTThingsGraphClient
    from types_boto3_iotthingsgraph.paginator import (
        GetFlowTemplateRevisionsPaginator,
        GetSystemTemplateRevisionsPaginator,
        ListFlowExecutionMessagesPaginator,
        ListTagsForResourcePaginator,
        SearchEntitiesPaginator,
        SearchFlowExecutionsPaginator,
        SearchFlowTemplatesPaginator,
        SearchSystemInstancesPaginator,
        SearchSystemTemplatesPaginator,
        SearchThingsPaginator,
    )

    session = Session()
    client: IoTThingsGraphClient = session.client("iotthingsgraph")

    get_flow_template_revisions_paginator: GetFlowTemplateRevisionsPaginator = client.get_paginator("get_flow_template_revisions")
    get_system_template_revisions_paginator: GetSystemTemplateRevisionsPaginator = client.get_paginator("get_system_template_revisions")
    list_flow_execution_messages_paginator: ListFlowExecutionMessagesPaginator = client.get_paginator("list_flow_execution_messages")
    list_tags_for_resource_paginator: ListTagsForResourcePaginator = client.get_paginator("list_tags_for_resource")
    search_entities_paginator: SearchEntitiesPaginator = client.get_paginator("search_entities")
    search_flow_executions_paginator: SearchFlowExecutionsPaginator = client.get_paginator("search_flow_executions")
    search_flow_templates_paginator: SearchFlowTemplatesPaginator = client.get_paginator("search_flow_templates")
    search_system_instances_paginator: SearchSystemInstancesPaginator = client.get_paginator("search_system_instances")
    search_system_templates_paginator: SearchSystemTemplatesPaginator = client.get_paginator("search_system_templates")
    search_things_paginator: SearchThingsPaginator = client.get_paginator("search_things")
    ```
"""

from __future__ import annotations

import sys
from typing import TYPE_CHECKING

from botocore.paginate import PageIterator, Paginator

from .type_defs import (
    GetFlowTemplateRevisionsRequestPaginateTypeDef,
    GetFlowTemplateRevisionsResponseTypeDef,
    GetSystemTemplateRevisionsRequestPaginateTypeDef,
    GetSystemTemplateRevisionsResponseTypeDef,
    ListFlowExecutionMessagesRequestPaginateTypeDef,
    ListFlowExecutionMessagesResponseTypeDef,
    ListTagsForResourceRequestPaginateTypeDef,
    ListTagsForResourceResponseTypeDef,
    SearchEntitiesRequestPaginateTypeDef,
    SearchEntitiesResponseTypeDef,
    SearchFlowExecutionsRequestPaginateTypeDef,
    SearchFlowExecutionsResponseTypeDef,
    SearchFlowTemplatesRequestPaginateTypeDef,
    SearchFlowTemplatesResponseTypeDef,
    SearchSystemInstancesRequestPaginateTypeDef,
    SearchSystemInstancesResponseTypeDef,
    SearchSystemTemplatesRequestPaginateTypeDef,
    SearchSystemTemplatesResponseTypeDef,
    SearchThingsRequestPaginateTypeDef,
    SearchThingsResponseTypeDef,
)

if sys.version_info >= (3, 12):
    from typing import Unpack
else:
    from typing_extensions import Unpack

__all__ = (
    "GetFlowTemplateRevisionsPaginator",
    "GetSystemTemplateRevisionsPaginator",
    "ListFlowExecutionMessagesPaginator",
    "ListTagsForResourcePaginator",
    "SearchEntitiesPaginator",
    "SearchFlowExecutionsPaginator",
    "SearchFlowTemplatesPaginator",
    "SearchSystemInstancesPaginator",
    "SearchSystemTemplatesPaginator",
    "SearchThingsPaginator",
)

if TYPE_CHECKING:
    _GetFlowTemplateRevisionsPaginatorBase = Paginator[GetFlowTemplateRevisionsResponseTypeDef]
else:
    _GetFlowTemplateRevisionsPaginatorBase = Paginator  # type: ignore[assignment]

class GetFlowTemplateRevisionsPaginator(_GetFlowTemplateRevisionsPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/GetFlowTemplateRevisions.html#IoTThingsGraph.Paginator.GetFlowTemplateRevisions)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#getflowtemplaterevisionspaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[GetFlowTemplateRevisionsRequestPaginateTypeDef]
    ) -> PageIterator[GetFlowTemplateRevisionsResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/GetFlowTemplateRevisions.html#IoTThingsGraph.Paginator.GetFlowTemplateRevisions.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#getflowtemplaterevisionspaginator)
        """

if TYPE_CHECKING:
    _GetSystemTemplateRevisionsPaginatorBase = Paginator[GetSystemTemplateRevisionsResponseTypeDef]
else:
    _GetSystemTemplateRevisionsPaginatorBase = Paginator  # type: ignore[assignment]

class GetSystemTemplateRevisionsPaginator(_GetSystemTemplateRevisionsPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/GetSystemTemplateRevisions.html#IoTThingsGraph.Paginator.GetSystemTemplateRevisions)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#getsystemtemplaterevisionspaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[GetSystemTemplateRevisionsRequestPaginateTypeDef]
    ) -> PageIterator[GetSystemTemplateRevisionsResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/GetSystemTemplateRevisions.html#IoTThingsGraph.Paginator.GetSystemTemplateRevisions.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#getsystemtemplaterevisionspaginator)
        """

if TYPE_CHECKING:
    _ListFlowExecutionMessagesPaginatorBase = Paginator[ListFlowExecutionMessagesResponseTypeDef]
else:
    _ListFlowExecutionMessagesPaginatorBase = Paginator  # type: ignore[assignment]

class ListFlowExecutionMessagesPaginator(_ListFlowExecutionMessagesPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/ListFlowExecutionMessages.html#IoTThingsGraph.Paginator.ListFlowExecutionMessages)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#listflowexecutionmessagespaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[ListFlowExecutionMessagesRequestPaginateTypeDef]
    ) -> PageIterator[ListFlowExecutionMessagesResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/ListFlowExecutionMessages.html#IoTThingsGraph.Paginator.ListFlowExecutionMessages.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#listflowexecutionmessagespaginator)
        """

if TYPE_CHECKING:
    _ListTagsForResourcePaginatorBase = Paginator[ListTagsForResourceResponseTypeDef]
else:
    _ListTagsForResourcePaginatorBase = Paginator  # type: ignore[assignment]

class ListTagsForResourcePaginator(_ListTagsForResourcePaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/ListTagsForResource.html#IoTThingsGraph.Paginator.ListTagsForResource)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#listtagsforresourcepaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[ListTagsForResourceRequestPaginateTypeDef]
    ) -> PageIterator[ListTagsForResourceResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/ListTagsForResource.html#IoTThingsGraph.Paginator.ListTagsForResource.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#listtagsforresourcepaginator)
        """

if TYPE_CHECKING:
    _SearchEntitiesPaginatorBase = Paginator[SearchEntitiesResponseTypeDef]
else:
    _SearchEntitiesPaginatorBase = Paginator  # type: ignore[assignment]

class SearchEntitiesPaginator(_SearchEntitiesPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchEntities.html#IoTThingsGraph.Paginator.SearchEntities)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchentitiespaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[SearchEntitiesRequestPaginateTypeDef]
    ) -> PageIterator[SearchEntitiesResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchEntities.html#IoTThingsGraph.Paginator.SearchEntities.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchentitiespaginator)
        """

if TYPE_CHECKING:
    _SearchFlowExecutionsPaginatorBase = Paginator[SearchFlowExecutionsResponseTypeDef]
else:
    _SearchFlowExecutionsPaginatorBase = Paginator  # type: ignore[assignment]

class SearchFlowExecutionsPaginator(_SearchFlowExecutionsPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchFlowExecutions.html#IoTThingsGraph.Paginator.SearchFlowExecutions)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchflowexecutionspaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[SearchFlowExecutionsRequestPaginateTypeDef]
    ) -> PageIterator[SearchFlowExecutionsResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchFlowExecutions.html#IoTThingsGraph.Paginator.SearchFlowExecutions.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchflowexecutionspaginator)
        """

if TYPE_CHECKING:
    _SearchFlowTemplatesPaginatorBase = Paginator[SearchFlowTemplatesResponseTypeDef]
else:
    _SearchFlowTemplatesPaginatorBase = Paginator  # type: ignore[assignment]

class SearchFlowTemplatesPaginator(_SearchFlowTemplatesPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchFlowTemplates.html#IoTThingsGraph.Paginator.SearchFlowTemplates)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchflowtemplatespaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[SearchFlowTemplatesRequestPaginateTypeDef]
    ) -> PageIterator[SearchFlowTemplatesResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchFlowTemplates.html#IoTThingsGraph.Paginator.SearchFlowTemplates.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchflowtemplatespaginator)
        """

if TYPE_CHECKING:
    _SearchSystemInstancesPaginatorBase = Paginator[SearchSystemInstancesResponseTypeDef]
else:
    _SearchSystemInstancesPaginatorBase = Paginator  # type: ignore[assignment]

class SearchSystemInstancesPaginator(_SearchSystemInstancesPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchSystemInstances.html#IoTThingsGraph.Paginator.SearchSystemInstances)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchsysteminstancespaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[SearchSystemInstancesRequestPaginateTypeDef]
    ) -> PageIterator[SearchSystemInstancesResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchSystemInstances.html#IoTThingsGraph.Paginator.SearchSystemInstances.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchsysteminstancespaginator)
        """

if TYPE_CHECKING:
    _SearchSystemTemplatesPaginatorBase = Paginator[SearchSystemTemplatesResponseTypeDef]
else:
    _SearchSystemTemplatesPaginatorBase = Paginator  # type: ignore[assignment]

class SearchSystemTemplatesPaginator(_SearchSystemTemplatesPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchSystemTemplates.html#IoTThingsGraph.Paginator.SearchSystemTemplates)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchsystemtemplatespaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[SearchSystemTemplatesRequestPaginateTypeDef]
    ) -> PageIterator[SearchSystemTemplatesResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchSystemTemplates.html#IoTThingsGraph.Paginator.SearchSystemTemplates.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchsystemtemplatespaginator)
        """

if TYPE_CHECKING:
    _SearchThingsPaginatorBase = Paginator[SearchThingsResponseTypeDef]
else:
    _SearchThingsPaginatorBase = Paginator  # type: ignore[assignment]

class SearchThingsPaginator(_SearchThingsPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchThings.html#IoTThingsGraph.Paginator.SearchThings)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchthingspaginator)
    """
    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[SearchThingsRequestPaginateTypeDef]
    ) -> PageIterator[SearchThingsResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/paginator/SearchThings.html#IoTThingsGraph.Paginator.SearchThings.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/paginators/#searchthingspaginator)
        """



"""
Type annotations for iotthingsgraph service type definitions.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/type_defs/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from types_boto3_iotthingsgraph.type_defs import AssociateEntityToThingRequestTypeDef

    data: AssociateEntityToThingRequestTypeDef = ...
    ```
"""

from __future__ import annotations

import sys
from datetime import datetime
from typing import Union

from .literals import (
    DeploymentTargetType,
    EntityFilterNameType,
    EntityTypeType,
    FlowExecutionEventTypeType,
    FlowExecutionStatusType,
    NamespaceDeletionStatusType,
    SystemInstanceDeploymentStatusType,
    SystemInstanceFilterNameType,
    UploadStatusType,
)

if sys.version_info >= (3, 9):
    from builtins import dict as Dict
    from builtins import list as List
    from collections.abc import Sequence
else:
    from typing import Dict, List, Sequence
if sys.version_info >= (3, 12):
    from typing import Literal, NotRequired, TypedDict
else:
    from typing_extensions import Literal, NotRequired, TypedDict


__all__ = (
    "AssociateEntityToThingRequestTypeDef",
    "CreateFlowTemplateRequestTypeDef",
    "CreateFlowTemplateResponseTypeDef",
    "CreateSystemInstanceRequestTypeDef",
    "CreateSystemInstanceResponseTypeDef",
    "CreateSystemTemplateRequestTypeDef",
    "CreateSystemTemplateResponseTypeDef",
    "DefinitionDocumentTypeDef",
    "DeleteFlowTemplateRequestTypeDef",
    "DeleteNamespaceResponseTypeDef",
    "DeleteSystemInstanceRequestTypeDef",
    "DeleteSystemTemplateRequestTypeDef",
    "DependencyRevisionTypeDef",
    "DeploySystemInstanceRequestTypeDef",
    "DeploySystemInstanceResponseTypeDef",
    "DeprecateFlowTemplateRequestTypeDef",
    "DeprecateSystemTemplateRequestTypeDef",
    "DescribeNamespaceRequestTypeDef",
    "DescribeNamespaceResponseTypeDef",
    "DissociateEntityFromThingRequestTypeDef",
    "EntityDescriptionTypeDef",
    "EntityFilterTypeDef",
    "FlowExecutionMessageTypeDef",
    "FlowExecutionSummaryTypeDef",
    "FlowTemplateDescriptionTypeDef",
    "FlowTemplateFilterTypeDef",
    "FlowTemplateSummaryTypeDef",
    "GetEntitiesRequestTypeDef",
    "GetEntitiesResponseTypeDef",
    "GetFlowTemplateRequestTypeDef",
    "GetFlowTemplateResponseTypeDef",
    "GetFlowTemplateRevisionsRequestPaginateTypeDef",
    "GetFlowTemplateRevisionsRequestTypeDef",
    "GetFlowTemplateRevisionsResponseTypeDef",
    "GetNamespaceDeletionStatusResponseTypeDef",
    "GetSystemInstanceRequestTypeDef",
    "GetSystemInstanceResponseTypeDef",
    "GetSystemTemplateRequestTypeDef",
    "GetSystemTemplateResponseTypeDef",
    "GetSystemTemplateRevisionsRequestPaginateTypeDef",
    "GetSystemTemplateRevisionsRequestTypeDef",
    "GetSystemTemplateRevisionsResponseTypeDef",
    "GetUploadStatusRequestTypeDef",
    "GetUploadStatusResponseTypeDef",
    "ListFlowExecutionMessagesRequestPaginateTypeDef",
    "ListFlowExecutionMessagesRequestTypeDef",
    "ListFlowExecutionMessagesResponseTypeDef",
    "ListTagsForResourceRequestPaginateTypeDef",
    "ListTagsForResourceRequestTypeDef",
    "ListTagsForResourceResponseTypeDef",
    "MetricsConfigurationTypeDef",
    "PaginatorConfigTypeDef",
    "ResponseMetadataTypeDef",
    "SearchEntitiesRequestPaginateTypeDef",
    "SearchEntitiesRequestTypeDef",
    "SearchEntitiesResponseTypeDef",
    "SearchFlowExecutionsRequestPaginateTypeDef",
    "SearchFlowExecutionsRequestTypeDef",
    "SearchFlowExecutionsResponseTypeDef",
    "SearchFlowTemplatesRequestPaginateTypeDef",
    "SearchFlowTemplatesRequestTypeDef",
    "SearchFlowTemplatesResponseTypeDef",
    "SearchSystemInstancesRequestPaginateTypeDef",
    "SearchSystemInstancesRequestTypeDef",
    "SearchSystemInstancesResponseTypeDef",
    "SearchSystemTemplatesRequestPaginateTypeDef",
    "SearchSystemTemplatesRequestTypeDef",
    "SearchSystemTemplatesResponseTypeDef",
    "SearchThingsRequestPaginateTypeDef",
    "SearchThingsRequestTypeDef",
    "SearchThingsResponseTypeDef",
    "SystemInstanceDescriptionTypeDef",
    "SystemInstanceFilterTypeDef",
    "SystemInstanceSummaryTypeDef",
    "SystemTemplateDescriptionTypeDef",
    "SystemTemplateFilterTypeDef",
    "SystemTemplateSummaryTypeDef",
    "TagResourceRequestTypeDef",
    "TagTypeDef",
    "ThingTypeDef",
    "TimestampTypeDef",
    "UndeploySystemInstanceRequestTypeDef",
    "UndeploySystemInstanceResponseTypeDef",
    "UntagResourceRequestTypeDef",
    "UpdateFlowTemplateRequestTypeDef",
    "UpdateFlowTemplateResponseTypeDef",
    "UpdateSystemTemplateRequestTypeDef",
    "UpdateSystemTemplateResponseTypeDef",
    "UploadEntityDefinitionsRequestTypeDef",
    "UploadEntityDefinitionsResponseTypeDef",
)


class AssociateEntityToThingRequestTypeDef(TypedDict):
    thingName: str
    entityId: str
    namespaceVersion: NotRequired[int]


class DefinitionDocumentTypeDef(TypedDict):
    language: Literal["GRAPHQL"]
    text: str


FlowTemplateSummaryTypeDef = TypedDict(
    "FlowTemplateSummaryTypeDef",
    {
        "id": NotRequired[str],
        "arn": NotRequired[str],
        "revisionNumber": NotRequired[int],
        "createdAt": NotRequired[datetime],
    },
)


class ResponseMetadataTypeDef(TypedDict):
    RequestId: str
    HTTPStatusCode: int
    HTTPHeaders: Dict[str, str]
    RetryAttempts: int
    HostId: NotRequired[str]


class MetricsConfigurationTypeDef(TypedDict):
    cloudMetricEnabled: NotRequired[bool]
    metricRuleRoleArn: NotRequired[str]


class TagTypeDef(TypedDict):
    key: str
    value: str


SystemInstanceSummaryTypeDef = TypedDict(
    "SystemInstanceSummaryTypeDef",
    {
        "id": NotRequired[str],
        "arn": NotRequired[str],
        "status": NotRequired[SystemInstanceDeploymentStatusType],
        "target": NotRequired[DeploymentTargetType],
        "greengrassGroupName": NotRequired[str],
        "createdAt": NotRequired[datetime],
        "updatedAt": NotRequired[datetime],
        "greengrassGroupId": NotRequired[str],
        "greengrassGroupVersionId": NotRequired[str],
    },
)
SystemTemplateSummaryTypeDef = TypedDict(
    "SystemTemplateSummaryTypeDef",
    {
        "id": NotRequired[str],
        "arn": NotRequired[str],
        "revisionNumber": NotRequired[int],
        "createdAt": NotRequired[datetime],
    },
)
DeleteFlowTemplateRequestTypeDef = TypedDict(
    "DeleteFlowTemplateRequestTypeDef",
    {
        "id": str,
    },
)
DeleteSystemInstanceRequestTypeDef = TypedDict(
    "DeleteSystemInstanceRequestTypeDef",
    {
        "id": NotRequired[str],
    },
)
DeleteSystemTemplateRequestTypeDef = TypedDict(
    "DeleteSystemTemplateRequestTypeDef",
    {
        "id": str,
    },
)
DependencyRevisionTypeDef = TypedDict(
    "DependencyRevisionTypeDef",
    {
        "id": NotRequired[str],
        "revisionNumber": NotRequired[int],
    },
)
DeploySystemInstanceRequestTypeDef = TypedDict(
    "DeploySystemInstanceRequestTypeDef",
    {
        "id": NotRequired[str],
    },
)
DeprecateFlowTemplateRequestTypeDef = TypedDict(
    "DeprecateFlowTemplateRequestTypeDef",
    {
        "id": str,
    },
)
DeprecateSystemTemplateRequestTypeDef = TypedDict(
    "DeprecateSystemTemplateRequestTypeDef",
    {
        "id": str,
    },
)


class DescribeNamespaceRequestTypeDef(TypedDict):
    namespaceName: NotRequired[str]


class DissociateEntityFromThingRequestTypeDef(TypedDict):
    thingName: str
    entityType: EntityTypeType


class EntityFilterTypeDef(TypedDict):
    name: NotRequired[EntityFilterNameType]
    value: NotRequired[Sequence[str]]


class FlowExecutionMessageTypeDef(TypedDict):
    messageId: NotRequired[str]
    eventType: NotRequired[FlowExecutionEventTypeType]
    timestamp: NotRequired[datetime]
    payload: NotRequired[str]


class FlowExecutionSummaryTypeDef(TypedDict):
    flowExecutionId: NotRequired[str]
    status: NotRequired[FlowExecutionStatusType]
    systemInstanceId: NotRequired[str]
    flowTemplateId: NotRequired[str]
    createdAt: NotRequired[datetime]
    updatedAt: NotRequired[datetime]


class FlowTemplateFilterTypeDef(TypedDict):
    name: Literal["DEVICE_MODEL_ID"]
    value: Sequence[str]


class GetEntitiesRequestTypeDef(TypedDict):
    ids: Sequence[str]
    namespaceVersion: NotRequired[int]


GetFlowTemplateRequestTypeDef = TypedDict(
    "GetFlowTemplateRequestTypeDef",
    {
        "id": str,
        "revisionNumber": NotRequired[int],
    },
)


class PaginatorConfigTypeDef(TypedDict):
    MaxItems: NotRequired[int]
    PageSize: NotRequired[int]
    StartingToken: NotRequired[str]


GetFlowTemplateRevisionsRequestTypeDef = TypedDict(
    "GetFlowTemplateRevisionsRequestTypeDef",
    {
        "id": str,
        "nextToken": NotRequired[str],
        "maxResults": NotRequired[int],
    },
)
GetSystemInstanceRequestTypeDef = TypedDict(
    "GetSystemInstanceRequestTypeDef",
    {
        "id": str,
    },
)
GetSystemTemplateRequestTypeDef = TypedDict(
    "GetSystemTemplateRequestTypeDef",
    {
        "id": str,
        "revisionNumber": NotRequired[int],
    },
)
GetSystemTemplateRevisionsRequestTypeDef = TypedDict(
    "GetSystemTemplateRevisionsRequestTypeDef",
    {
        "id": str,
        "nextToken": NotRequired[str],
        "maxResults": NotRequired[int],
    },
)


class GetUploadStatusRequestTypeDef(TypedDict):
    uploadId: str


class ListFlowExecutionMessagesRequestTypeDef(TypedDict):
    flowExecutionId: str
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]


class ListTagsForResourceRequestTypeDef(TypedDict):
    resourceArn: str
    maxResults: NotRequired[int]
    nextToken: NotRequired[str]


TimestampTypeDef = Union[datetime, str]


class SystemInstanceFilterTypeDef(TypedDict):
    name: NotRequired[SystemInstanceFilterNameType]
    value: NotRequired[Sequence[str]]


class SystemTemplateFilterTypeDef(TypedDict):
    name: Literal["FLOW_TEMPLATE_ID"]
    value: Sequence[str]


class SearchThingsRequestTypeDef(TypedDict):
    entityId: str
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]
    namespaceVersion: NotRequired[int]


class ThingTypeDef(TypedDict):
    thingArn: NotRequired[str]
    thingName: NotRequired[str]


UndeploySystemInstanceRequestTypeDef = TypedDict(
    "UndeploySystemInstanceRequestTypeDef",
    {
        "id": NotRequired[str],
    },
)


class UntagResourceRequestTypeDef(TypedDict):
    resourceArn: str
    tagKeys: Sequence[str]


class CreateFlowTemplateRequestTypeDef(TypedDict):
    definition: DefinitionDocumentTypeDef
    compatibleNamespaceVersion: NotRequired[int]


class CreateSystemTemplateRequestTypeDef(TypedDict):
    definition: DefinitionDocumentTypeDef
    compatibleNamespaceVersion: NotRequired[int]


EntityDescriptionTypeDef = TypedDict(
    "EntityDescriptionTypeDef",
    {
        "id": NotRequired[str],
        "arn": NotRequired[str],
        "type": NotRequired[EntityTypeType],
        "createdAt": NotRequired[datetime],
        "definition": NotRequired[DefinitionDocumentTypeDef],
    },
)
UpdateFlowTemplateRequestTypeDef = TypedDict(
    "UpdateFlowTemplateRequestTypeDef",
    {
        "id": str,
        "definition": DefinitionDocumentTypeDef,
        "compatibleNamespaceVersion": NotRequired[int],
    },
)
UpdateSystemTemplateRequestTypeDef = TypedDict(
    "UpdateSystemTemplateRequestTypeDef",
    {
        "id": str,
        "definition": DefinitionDocumentTypeDef,
        "compatibleNamespaceVersion": NotRequired[int],
    },
)


class UploadEntityDefinitionsRequestTypeDef(TypedDict):
    document: NotRequired[DefinitionDocumentTypeDef]
    syncWithPublicNamespace: NotRequired[bool]
    deprecateExistingEntities: NotRequired[bool]


class FlowTemplateDescriptionTypeDef(TypedDict):
    summary: NotRequired[FlowTemplateSummaryTypeDef]
    definition: NotRequired[DefinitionDocumentTypeDef]
    validatedNamespaceVersion: NotRequired[int]


class CreateFlowTemplateResponseTypeDef(TypedDict):
    summary: FlowTemplateSummaryTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class DeleteNamespaceResponseTypeDef(TypedDict):
    namespaceArn: str
    namespaceName: str
    ResponseMetadata: ResponseMetadataTypeDef


class DescribeNamespaceResponseTypeDef(TypedDict):
    namespaceArn: str
    namespaceName: str
    trackingNamespaceName: str
    trackingNamespaceVersion: int
    namespaceVersion: int
    ResponseMetadata: ResponseMetadataTypeDef


class GetFlowTemplateRevisionsResponseTypeDef(TypedDict):
    summaries: List[FlowTemplateSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class GetNamespaceDeletionStatusResponseTypeDef(TypedDict):
    namespaceArn: str
    namespaceName: str
    status: NamespaceDeletionStatusType
    errorCode: Literal["VALIDATION_FAILED"]
    errorMessage: str
    ResponseMetadata: ResponseMetadataTypeDef


class GetUploadStatusResponseTypeDef(TypedDict):
    uploadId: str
    uploadStatus: UploadStatusType
    namespaceArn: str
    namespaceName: str
    namespaceVersion: int
    failureReason: List[str]
    createdDate: datetime
    ResponseMetadata: ResponseMetadataTypeDef


class SearchFlowTemplatesResponseTypeDef(TypedDict):
    summaries: List[FlowTemplateSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class UpdateFlowTemplateResponseTypeDef(TypedDict):
    summary: FlowTemplateSummaryTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class UploadEntityDefinitionsResponseTypeDef(TypedDict):
    uploadId: str
    ResponseMetadata: ResponseMetadataTypeDef


class CreateSystemInstanceRequestTypeDef(TypedDict):
    definition: DefinitionDocumentTypeDef
    target: DeploymentTargetType
    tags: NotRequired[Sequence[TagTypeDef]]
    greengrassGroupName: NotRequired[str]
    s3BucketName: NotRequired[str]
    metricsConfiguration: NotRequired[MetricsConfigurationTypeDef]
    flowActionsRoleArn: NotRequired[str]


class ListTagsForResourceResponseTypeDef(TypedDict):
    tags: List[TagTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class TagResourceRequestTypeDef(TypedDict):
    resourceArn: str
    tags: Sequence[TagTypeDef]


class CreateSystemInstanceResponseTypeDef(TypedDict):
    summary: SystemInstanceSummaryTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class DeploySystemInstanceResponseTypeDef(TypedDict):
    summary: SystemInstanceSummaryTypeDef
    greengrassDeploymentId: str
    ResponseMetadata: ResponseMetadataTypeDef


class SearchSystemInstancesResponseTypeDef(TypedDict):
    summaries: List[SystemInstanceSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class UndeploySystemInstanceResponseTypeDef(TypedDict):
    summary: SystemInstanceSummaryTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class CreateSystemTemplateResponseTypeDef(TypedDict):
    summary: SystemTemplateSummaryTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class GetSystemTemplateRevisionsResponseTypeDef(TypedDict):
    summaries: List[SystemTemplateSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class SearchSystemTemplatesResponseTypeDef(TypedDict):
    summaries: List[SystemTemplateSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class SystemTemplateDescriptionTypeDef(TypedDict):
    summary: NotRequired[SystemTemplateSummaryTypeDef]
    definition: NotRequired[DefinitionDocumentTypeDef]
    validatedNamespaceVersion: NotRequired[int]


class UpdateSystemTemplateResponseTypeDef(TypedDict):
    summary: SystemTemplateSummaryTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class SystemInstanceDescriptionTypeDef(TypedDict):
    summary: NotRequired[SystemInstanceSummaryTypeDef]
    definition: NotRequired[DefinitionDocumentTypeDef]
    s3BucketName: NotRequired[str]
    metricsConfiguration: NotRequired[MetricsConfigurationTypeDef]
    validatedNamespaceVersion: NotRequired[int]
    validatedDependencyRevisions: NotRequired[List[DependencyRevisionTypeDef]]
    flowActionsRoleArn: NotRequired[str]


class SearchEntitiesRequestTypeDef(TypedDict):
    entityTypes: Sequence[EntityTypeType]
    filters: NotRequired[Sequence[EntityFilterTypeDef]]
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]
    namespaceVersion: NotRequired[int]


class ListFlowExecutionMessagesResponseTypeDef(TypedDict):
    messages: List[FlowExecutionMessageTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class SearchFlowExecutionsResponseTypeDef(TypedDict):
    summaries: List[FlowExecutionSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class SearchFlowTemplatesRequestTypeDef(TypedDict):
    filters: NotRequired[Sequence[FlowTemplateFilterTypeDef]]
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]


GetFlowTemplateRevisionsRequestPaginateTypeDef = TypedDict(
    "GetFlowTemplateRevisionsRequestPaginateTypeDef",
    {
        "id": str,
        "PaginationConfig": NotRequired[PaginatorConfigTypeDef],
    },
)
GetSystemTemplateRevisionsRequestPaginateTypeDef = TypedDict(
    "GetSystemTemplateRevisionsRequestPaginateTypeDef",
    {
        "id": str,
        "PaginationConfig": NotRequired[PaginatorConfigTypeDef],
    },
)


class ListFlowExecutionMessagesRequestPaginateTypeDef(TypedDict):
    flowExecutionId: str
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class ListTagsForResourceRequestPaginateTypeDef(TypedDict):
    resourceArn: str
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchEntitiesRequestPaginateTypeDef(TypedDict):
    entityTypes: Sequence[EntityTypeType]
    filters: NotRequired[Sequence[EntityFilterTypeDef]]
    namespaceVersion: NotRequired[int]
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchFlowTemplatesRequestPaginateTypeDef(TypedDict):
    filters: NotRequired[Sequence[FlowTemplateFilterTypeDef]]
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchThingsRequestPaginateTypeDef(TypedDict):
    entityId: str
    namespaceVersion: NotRequired[int]
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchFlowExecutionsRequestPaginateTypeDef(TypedDict):
    systemInstanceId: str
    flowExecutionId: NotRequired[str]
    startTime: NotRequired[TimestampTypeDef]
    endTime: NotRequired[TimestampTypeDef]
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchFlowExecutionsRequestTypeDef(TypedDict):
    systemInstanceId: str
    flowExecutionId: NotRequired[str]
    startTime: NotRequired[TimestampTypeDef]
    endTime: NotRequired[TimestampTypeDef]
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]


class SearchSystemInstancesRequestPaginateTypeDef(TypedDict):
    filters: NotRequired[Sequence[SystemInstanceFilterTypeDef]]
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchSystemInstancesRequestTypeDef(TypedDict):
    filters: NotRequired[Sequence[SystemInstanceFilterTypeDef]]
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]


class SearchSystemTemplatesRequestPaginateTypeDef(TypedDict):
    filters: NotRequired[Sequence[SystemTemplateFilterTypeDef]]
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]


class SearchSystemTemplatesRequestTypeDef(TypedDict):
    filters: NotRequired[Sequence[SystemTemplateFilterTypeDef]]
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]


class SearchThingsResponseTypeDef(TypedDict):
    things: List[ThingTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class GetEntitiesResponseTypeDef(TypedDict):
    descriptions: List[EntityDescriptionTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef


class SearchEntitiesResponseTypeDef(TypedDict):
    descriptions: List[EntityDescriptionTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]


class GetFlowTemplateResponseTypeDef(TypedDict):
    description: FlowTemplateDescriptionTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class GetSystemTemplateResponseTypeDef(TypedDict):
    description: SystemTemplateDescriptionTypeDef
    ResponseMetadata: ResponseMetadataTypeDef


class GetSystemInstanceResponseTypeDef(TypedDict):
    description: SystemInstanceDescriptionTypeDef
    ResponseMetadata: ResponseMetadataTypeDef



"""
Type annotations for iotthingsgraph service Client.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from boto3.session import Session
    from types_boto3_iotthingsgraph.client import IoTThingsGraphClient

    session = Session()
    client: IoTThingsGraphClient = session.client("iotthingsgraph")
    ```
"""

from __future__ import annotations

import sys
from typing import Any, overload

from botocore.client import BaseClient, ClientMeta
from botocore.errorfactory import BaseClientExceptions
from botocore.exceptions import ClientError as BotocoreClientError

from .paginator import (
    GetFlowTemplateRevisionsPaginator,
    GetSystemTemplateRevisionsPaginator,
    ListFlowExecutionMessagesPaginator,
    ListTagsForResourcePaginator,
    SearchEntitiesPaginator,
    SearchFlowExecutionsPaginator,
    SearchFlowTemplatesPaginator,
    SearchSystemInstancesPaginator,
    SearchSystemTemplatesPaginator,
    SearchThingsPaginator,
)
from .type_defs import (
    AssociateEntityToThingRequestTypeDef,
    CreateFlowTemplateRequestTypeDef,
    CreateFlowTemplateResponseTypeDef,
    CreateSystemInstanceRequestTypeDef,
    CreateSystemInstanceResponseTypeDef,
    CreateSystemTemplateRequestTypeDef,
    CreateSystemTemplateResponseTypeDef,
    DeleteFlowTemplateRequestTypeDef,
    DeleteNamespaceResponseTypeDef,
    DeleteSystemInstanceRequestTypeDef,
    DeleteSystemTemplateRequestTypeDef,
    DeploySystemInstanceRequestTypeDef,
    DeploySystemInstanceResponseTypeDef,
    DeprecateFlowTemplateRequestTypeDef,
    DeprecateSystemTemplateRequestTypeDef,
    DescribeNamespaceRequestTypeDef,
    DescribeNamespaceResponseTypeDef,
    DissociateEntityFromThingRequestTypeDef,
    GetEntitiesRequestTypeDef,
    GetEntitiesResponseTypeDef,
    GetFlowTemplateRequestTypeDef,
    GetFlowTemplateResponseTypeDef,
    GetFlowTemplateRevisionsRequestTypeDef,
    GetFlowTemplateRevisionsResponseTypeDef,
    GetNamespaceDeletionStatusResponseTypeDef,
    GetSystemInstanceRequestTypeDef,
    GetSystemInstanceResponseTypeDef,
    GetSystemTemplateRequestTypeDef,
    GetSystemTemplateResponseTypeDef,
    GetSystemTemplateRevisionsRequestTypeDef,
    GetSystemTemplateRevisionsResponseTypeDef,
    GetUploadStatusRequestTypeDef,
    GetUploadStatusResponseTypeDef,
    ListFlowExecutionMessagesRequestTypeDef,
    ListFlowExecutionMessagesResponseTypeDef,
    ListTagsForResourceRequestTypeDef,
    ListTagsForResourceResponseTypeDef,
    SearchEntitiesRequestTypeDef,
    SearchEntitiesResponseTypeDef,
    SearchFlowExecutionsRequestTypeDef,
    SearchFlowExecutionsResponseTypeDef,
    SearchFlowTemplatesRequestTypeDef,
    SearchFlowTemplatesResponseTypeDef,
    SearchSystemInstancesRequestTypeDef,
    SearchSystemInstancesResponseTypeDef,
    SearchSystemTemplatesRequestTypeDef,
    SearchSystemTemplatesResponseTypeDef,
    SearchThingsRequestTypeDef,
    SearchThingsResponseTypeDef,
    TagResourceRequestTypeDef,
    UndeploySystemInstanceRequestTypeDef,
    UndeploySystemInstanceResponseTypeDef,
    UntagResourceRequestTypeDef,
    UpdateFlowTemplateRequestTypeDef,
    UpdateFlowTemplateResponseTypeDef,
    UpdateSystemTemplateRequestTypeDef,
    UpdateSystemTemplateResponseTypeDef,
    UploadEntityDefinitionsRequestTypeDef,
    UploadEntityDefinitionsResponseTypeDef,
)

if sys.version_info >= (3, 9):
    from builtins import dict as Dict
    from builtins import type as Type
    from collections.abc import Mapping
else:
    from typing import Dict, Mapping, Type
if sys.version_info >= (3, 12):
    from typing import Literal, Unpack
else:
    from typing_extensions import Literal, Unpack

__all__ = ("IoTThingsGraphClient",)

class Exceptions(BaseClientExceptions):
    ClientError: Type[BotocoreClientError]
    InternalFailureException: Type[BotocoreClientError]
    InvalidRequestException: Type[BotocoreClientError]
    LimitExceededException: Type[BotocoreClientError]
    ResourceAlreadyExistsException: Type[BotocoreClientError]
    ResourceInUseException: Type[BotocoreClientError]
    ResourceNotFoundException: Type[BotocoreClientError]
    ThrottlingException: Type[BotocoreClientError]

class IoTThingsGraphClient(BaseClient):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph.html#IoTThingsGraph.Client)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/)
    """

    meta: ClientMeta

    @property
    def exceptions(self) -> Exceptions:
        """
        IoTThingsGraphClient exceptions.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph.html#IoTThingsGraph.Client)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#exceptions)
        """

    def can_paginate(self, operation_name: str) -> bool:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/can_paginate.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#can_paginate)
        """

    def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Mapping[str, Any] = ...,
        ExpiresIn: int = 3600,
        HttpMethod: str = ...,
    ) -> str:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/generate_presigned_url.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#generate_presigned_url)
        """

    def associate_entity_to_thing(
        self, **kwargs: Unpack[AssociateEntityToThingRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Associates a device with a concrete thing that is in the user's registry.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/associate_entity_to_thing.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#associate_entity_to_thing)
        """

    def create_flow_template(
        self, **kwargs: Unpack[CreateFlowTemplateRequestTypeDef]
    ) -> CreateFlowTemplateResponseTypeDef:
        """
        Creates a workflow template.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/create_flow_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#create_flow_template)
        """

    def create_system_instance(
        self, **kwargs: Unpack[CreateSystemInstanceRequestTypeDef]
    ) -> CreateSystemInstanceResponseTypeDef:
        """
        Creates a system instance.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/create_system_instance.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#create_system_instance)
        """

    def create_system_template(
        self, **kwargs: Unpack[CreateSystemTemplateRequestTypeDef]
    ) -> CreateSystemTemplateResponseTypeDef:
        """
        Creates a system.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/create_system_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#create_system_template)
        """

    def delete_flow_template(
        self, **kwargs: Unpack[DeleteFlowTemplateRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Deletes a workflow.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/delete_flow_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#delete_flow_template)
        """

    def delete_namespace(self) -> DeleteNamespaceResponseTypeDef:
        """
        Deletes the specified namespace.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/delete_namespace.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#delete_namespace)
        """

    def delete_system_instance(
        self, **kwargs: Unpack[DeleteSystemInstanceRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Deletes a system instance.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/delete_system_instance.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#delete_system_instance)
        """

    def delete_system_template(
        self, **kwargs: Unpack[DeleteSystemTemplateRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Deletes a system.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/delete_system_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#delete_system_template)
        """

    def deploy_system_instance(
        self, **kwargs: Unpack[DeploySystemInstanceRequestTypeDef]
    ) -> DeploySystemInstanceResponseTypeDef:
        """
        <b>Greengrass and Cloud Deployments</b>.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/deploy_system_instance.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#deploy_system_instance)
        """

    def deprecate_flow_template(
        self, **kwargs: Unpack[DeprecateFlowTemplateRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Deprecates the specified workflow.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/deprecate_flow_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#deprecate_flow_template)
        """

    def deprecate_system_template(
        self, **kwargs: Unpack[DeprecateSystemTemplateRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Deprecates the specified system.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/deprecate_system_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#deprecate_system_template)
        """

    def describe_namespace(
        self, **kwargs: Unpack[DescribeNamespaceRequestTypeDef]
    ) -> DescribeNamespaceResponseTypeDef:
        """
        Gets the latest version of the user's namespace and the public version that it
        is tracking.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/describe_namespace.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#describe_namespace)
        """

    def dissociate_entity_from_thing(
        self, **kwargs: Unpack[DissociateEntityFromThingRequestTypeDef]
    ) -> Dict[str, Any]:
        """
        Dissociates a device entity from a concrete thing.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/dissociate_entity_from_thing.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#dissociate_entity_from_thing)
        """

    def get_entities(
        self, **kwargs: Unpack[GetEntitiesRequestTypeDef]
    ) -> GetEntitiesResponseTypeDef:
        """
        Gets definitions of the specified entities.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_entities.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_entities)
        """

    def get_flow_template(
        self, **kwargs: Unpack[GetFlowTemplateRequestTypeDef]
    ) -> GetFlowTemplateResponseTypeDef:
        """
        Gets the latest version of the <code>DefinitionDocument</code> and
        <code>FlowTemplateSummary</code> for the specified workflow.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_flow_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_flow_template)
        """

    def get_flow_template_revisions(
        self, **kwargs: Unpack[GetFlowTemplateRevisionsRequestTypeDef]
    ) -> GetFlowTemplateRevisionsResponseTypeDef:
        """
        Gets revisions of the specified workflow.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_flow_template_revisions.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_flow_template_revisions)
        """

    def get_namespace_deletion_status(self) -> GetNamespaceDeletionStatusResponseTypeDef:
        """
        Gets the status of a namespace deletion task.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_namespace_deletion_status.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_namespace_deletion_status)
        """

    def get_system_instance(
        self, **kwargs: Unpack[GetSystemInstanceRequestTypeDef]
    ) -> GetSystemInstanceResponseTypeDef:
        """
        Gets a system instance.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_system_instance.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_system_instance)
        """

    def get_system_template(
        self, **kwargs: Unpack[GetSystemTemplateRequestTypeDef]
    ) -> GetSystemTemplateResponseTypeDef:
        """
        Gets a system.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_system_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_system_template)
        """

    def get_system_template_revisions(
        self, **kwargs: Unpack[GetSystemTemplateRevisionsRequestTypeDef]
    ) -> GetSystemTemplateRevisionsResponseTypeDef:
        """
        Gets revisions made to the specified system template.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_system_template_revisions.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_system_template_revisions)
        """

    def get_upload_status(
        self, **kwargs: Unpack[GetUploadStatusRequestTypeDef]
    ) -> GetUploadStatusResponseTypeDef:
        """
        Gets the status of the specified upload.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_upload_status.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_upload_status)
        """

    def list_flow_execution_messages(
        self, **kwargs: Unpack[ListFlowExecutionMessagesRequestTypeDef]
    ) -> ListFlowExecutionMessagesResponseTypeDef:
        """
        Returns a list of objects that contain information about events in a flow
        execution.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/list_flow_execution_messages.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#list_flow_execution_messages)
        """

    def list_tags_for_resource(
        self, **kwargs: Unpack[ListTagsForResourceRequestTypeDef]
    ) -> ListTagsForResourceResponseTypeDef:
        """
        Lists all tags on an AWS IoT Things Graph resource.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/list_tags_for_resource.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#list_tags_for_resource)
        """

    def search_entities(
        self, **kwargs: Unpack[SearchEntitiesRequestTypeDef]
    ) -> SearchEntitiesResponseTypeDef:
        """
        Searches for entities of the specified type.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/search_entities.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#search_entities)
        """

    def search_flow_executions(
        self, **kwargs: Unpack[SearchFlowExecutionsRequestTypeDef]
    ) -> SearchFlowExecutionsResponseTypeDef:
        """
        Searches for AWS IoT Things Graph workflow execution instances.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/search_flow_executions.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#search_flow_executions)
        """

    def search_flow_templates(
        self, **kwargs: Unpack[SearchFlowTemplatesRequestTypeDef]
    ) -> SearchFlowTemplatesResponseTypeDef:
        """
        Searches for summary information about workflows.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/search_flow_templates.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#search_flow_templates)
        """

    def search_system_instances(
        self, **kwargs: Unpack[SearchSystemInstancesRequestTypeDef]
    ) -> SearchSystemInstancesResponseTypeDef:
        """
        Searches for system instances in the user's account.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/search_system_instances.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#search_system_instances)
        """

    def search_system_templates(
        self, **kwargs: Unpack[SearchSystemTemplatesRequestTypeDef]
    ) -> SearchSystemTemplatesResponseTypeDef:
        """
        Searches for summary information about systems in the user's account.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/search_system_templates.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#search_system_templates)
        """

    def search_things(
        self, **kwargs: Unpack[SearchThingsRequestTypeDef]
    ) -> SearchThingsResponseTypeDef:
        """
        Searches for things associated with the specified entity.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/search_things.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#search_things)
        """

    def tag_resource(self, **kwargs: Unpack[TagResourceRequestTypeDef]) -> Dict[str, Any]:
        """
        Creates a tag for the specified resource.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/tag_resource.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#tag_resource)
        """

    def undeploy_system_instance(
        self, **kwargs: Unpack[UndeploySystemInstanceRequestTypeDef]
    ) -> UndeploySystemInstanceResponseTypeDef:
        """
        Removes a system instance from its target (Cloud or Greengrass).

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/undeploy_system_instance.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#undeploy_system_instance)
        """

    def untag_resource(self, **kwargs: Unpack[UntagResourceRequestTypeDef]) -> Dict[str, Any]:
        """
        Removes a tag from the specified resource.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/untag_resource.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#untag_resource)
        """

    def update_flow_template(
        self, **kwargs: Unpack[UpdateFlowTemplateRequestTypeDef]
    ) -> UpdateFlowTemplateResponseTypeDef:
        """
        Updates the specified workflow.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/update_flow_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#update_flow_template)
        """

    def update_system_template(
        self, **kwargs: Unpack[UpdateSystemTemplateRequestTypeDef]
    ) -> UpdateSystemTemplateResponseTypeDef:
        """
        Updates the specified system.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/update_system_template.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#update_system_template)
        """

    def upload_entity_definitions(
        self, **kwargs: Unpack[UploadEntityDefinitionsRequestTypeDef]
    ) -> UploadEntityDefinitionsResponseTypeDef:
        """
        Asynchronously uploads one or more entity definitions to the user's namespace.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/upload_entity_definitions.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#upload_entity_definitions)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["get_flow_template_revisions"]
    ) -> GetFlowTemplateRevisionsPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["get_system_template_revisions"]
    ) -> GetSystemTemplateRevisionsPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["list_flow_execution_messages"]
    ) -> ListFlowExecutionMessagesPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["list_tags_for_resource"]
    ) -> ListTagsForResourcePaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["search_entities"]
    ) -> SearchEntitiesPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["search_flow_executions"]
    ) -> SearchFlowExecutionsPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["search_flow_templates"]
    ) -> SearchFlowTemplatesPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["search_system_instances"]
    ) -> SearchSystemInstancesPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["search_system_templates"]
    ) -> SearchSystemTemplatesPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """

    @overload  # type: ignore[override]
    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["search_things"]
    ) -> SearchThingsPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iotthingsgraph/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iotthingsgraph/client/#get_paginator)
        """
